<?php
/**
 * Paygent Payment Gateway
 *
 * Provides a Paygent Credit Card Payment Gateway.
 *
 * @class 		WC_Paygent
 * @extends		WC_Gateway_Paygent_CC
 * @version		2.1.1
 * @package		WooCommerce/Classes/Payment
 * @author		Artisan Workshop
 */
class WC_Gateway_Paygent_CC_Addons extends WC_Gateway_Paygent_CC {

	/**
	 * Constructor for the gateway.
	 *
	 * @access public
	 * @return void
	 */
	public function __construct() {
		parent::__construct();

		if ( class_exists( 'WC_Subscriptions_Order' ) ) {
			add_action( 'woocommerce_scheduled_subscription_payment_' . $this->id, array( $this, 'scheduled_subscription_payment' ), 10, 2 );
		}
	}
	/**
	 * Check if order contains subscriptions.
	 *
	 * @param  int $order_id
	 * @return bool
	 */
	protected function order_contains_subscription( $order_id ) {
		return function_exists( 'wcs_order_contains_subscription' ) && ( wcs_order_contains_subscription( $order_id ) || wcs_order_contains_renewal( $order_id ) );
	}

	/**
	 * Is $order_id a subscription?
	 * @param  int  $order_id
	 * @return boolean
	 */
	protected function is_subscription( $order_id ) {
		return ( function_exists( 'wcs_order_contains_subscription' ) && ( wcs_order_contains_subscription( $order_id ) || wcs_is_subscription( $order_id ) || wcs_order_contains_renewal( $order_id ) ) );
	}

	/**
	 * Process the subscription.
	 *
	 * @param  WC_Order $order
	 * @param  boolean $subscription
	 * @return
	 */
	protected function process_subscription( $order , $subscription = false) {
		$payment_response = $this->process_subscription_payment( $order, $order->get_total() );
		return $payment_response;
	}

	/**
	 * Process the payment.
	 *
	 * @param  int $order_id
	 * @param  boolean $subscription
	 * @return array
	 */
	public function process_payment( $order_id , $subscription = false) {
		// Processing subscription
		if ( $this->is_subscription( $order_id ) ) {
			// Regular payment with force customer enabled
			return parent::process_payment( $order_id, true );
		} else {
			return parent::process_payment( $order_id, false );
		}
	}

	/**
	 * process_subscription_payment function.
	 *
	 * @param object WC_order $order
	 * @param int $amount (default: 0)
	 * @return bool|WP_Error
	 */
	public function process_subscription_payment( $order = '', $amount = 0 ) {
		if ( 0 == $amount ) {
			// Payment complete
			$order->payment_complete();

			return true;
		}
		$order_id = $order->get_id();
		$customer_id = $order->get_customer_id();
        $card_user_id = 'wc'.$customer_id;
		$send_data = array();
		//Common header
		$telegram_kind = '020';
        $prefix_order = get_option( 'wc-paygent-prefix_order' );
        if($prefix_order){
            $send_data['trading_id'] = $prefix_order.$order_id;
        }else{
            $send_data['trading_id'] = 'wc_'.$order_id;
        }
		$send_data['payment_id'] = '';

		$send_data['payment_amount'] = $order->get_total();

	  	//Payment times
		$send_data['payment_class'] = 10;//One time payment
		$send_data['3dsecure_ryaku'] = 1;
        $send_data['3dsecure_use_type'] = 1;
        $send_data['stock_card_mode'] = 1;
        $send_data['security_code_token'] = 0;
        $send_data['customer_id'] = $card_user_id;
        $default_token = WC_Payment_Tokens::get_customer_default_token( $customer_id );
        foreach($default_token->get_meta_data() as $data_key => $data_value) {
            $main_key = 'key';
            foreach ($data_value->get_data() as $meta_key => $meta_value) {
                if ($meta_key == 'key') {
                    $main_key = $meta_value;
                } elseif ($meta_key == 'value') {
                    $paygent_tokens[$main_key] = $meta_value;
                }
            }
        }
        if(isset($paygent_tokens)) $send_data['customer_card_id'] = $paygent_tokens['customer_card_id'];

		$response = $this->paygent_request->send_paygent_request($this->test_mode, $order, $telegram_kind, $send_data, $this->debug);

		// Check response
		if ( $response['result'] == 0 and $response['result_array']) {
			// Success
			$order->add_order_note( __( 'Subscription Credit Card Payment completed.' , 'woocommerce-for-paygent-payment-main' ) );
            $order->add_meta_data('_paygent_order_id', $send_data['trading_id'], true );
			//set transaction id for Paygent Order Number
			$order->payment_complete(wc_clean( $response['result_array'][0]['payment_id'] ));
            if(isset($this->paymentaction) and $this->paymentaction == 'sale' ){
                $telegram_kind = '022';
                $response_sale = $this->paygent_request->send_paygent_request($this->test_mode, $order, $telegram_kind, $send_data, $this->debug);
                if($response_sale['result'] != 0){
                    $this->paygent_request->error_response($response_sale, $order);
                }
            }
			return true;
		}else{
            // Failed
            $order->add_order_note( __( 'Subscription Credit Card Payment failed.' , 'woocommerce-for-paygent-payment-main' ) );
			return $this->paygent_request->error_response($response, $order);
		}
	}

	/**
	 * scheduled_subscription_payment function.
	 *
	 * @param float $amount_to_charge The amount to charge.
	 * @param WC_Order $renewal_order A WC_Order object created to record the renewal payment.
	 */
	public function scheduled_subscription_payment( $amount_to_charge, $renewal_order ) {
		$result = $this->process_subscription_payment( $renewal_order, $amount_to_charge );

		if ( is_wp_error( $result ) ) {
			$renewal_order->update_status( 'failed', sprintf( __( 'Paygent Transaction Failed (%s)', 'woocommerce' ), $result->get_error_message() ) );
		}
	}
}
