<?php
/**
 * Paygent Payment Gateway
 *
 * Provides a Paygent Carrier Payment Gateway.
 *
 * @class 		WC_Paygent
 * @extends		WC_Gateway_Paygent_Carrier
 * @version		2.1.0
 * @package		WooCommerce/Classes/Payment
 * @author		Artisan Workshop
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

use ArtisanWorkshop\WooCommerce\PluginFramework\v2_0_13 as Framework;

class WC_Gateway_Paygent_Carrier extends WC_Payment_Gateway {

    /**
     * Framework.
     *
     * @var object
     */
    public $jp4wc_framework;

    /**
     * Debug mode
     *
     * @var string
     */
    public $debug;

    /**
     * Test mode
     *
     * @var string
     */
    public $test_mode;

    /**
     * Set gmopg request class
     *
     * @var stdClass
     */
    public $paygent_request;

    /**
	 * Constructor for the gateway.
	 *
	 * @access public
	 * @return void
	 */
	public function __construct() {

		$this->id                = 'paygent_mb';
		$this->has_fields        = false;
		$this->order_button_text = sprintf(__( 'Proceed to %s', 'woocommerce-for-paygent-payment-main' ), __('Carrier Payment', 'woocommerce-for-paygent-payment-main' ));
		
        // Create plugin fields and settings
		$this->init_form_fields();
		$this->init_settings();
		$this->method_title       = sprintf(__( 'Paygent %s Gateway', 'woocommerce-for-paygent-payment-main' ), __('Carrier Payment', 'woocommerce-for-paygent-payment-main' ));
		$this->method_description = sprintf(__( 'Allows payments by Paygent %s in Japan.', 'woocommerce-for-paygent-payment-main' ), __('Carrier Payment', 'woocommerce-for-paygent-payment-main' ));
		$this->supports = array(
			'products',
			'refunds',
		);
        // When no save setting error at chackout page
		if(is_null($this->title)){
			$this->title = __( 'Please set this payment at Control Panel! ', 'woocommerce-for-paygent-payment-main' ).$this->method_title;
		}
		// Get setting values
		foreach ( $this->settings as $key => $val ) $this->$key = $val;

        //Set JP4WC framework
        $this->jp4wc_framework = new Framework\JP4WC_Plugin();

        include_once( 'includes/class-wc-gateway-paygent-request.php' );
        $this->paygent_request = new WC_Gateway_Paygent_Request();

        //Set Test mode.
        $this->test_mode = get_option('wc-paygent-testmode');

        // Set Carrier type
		$this->carrier_types = array();
		if($this->setting_ct_04 =='yes') $this->carrier_types = array_merge($this->carrier_types, array('04' => __( 'au Easy Payment', 'woocommerce-for-paygent-payment-main' )));
		if($this->setting_ct_05 =='yes') $this->carrier_types = array_merge($this->carrier_types, array('05' => __( 'Docomo Payment', 'woocommerce-for-paygent-payment-main' )));
		if($this->setting_ct_06 =='yes') $this->carrier_types = array_merge($this->carrier_types, array('06' => __( 'SoftBank Matomete Payment(B)', 'woocommerce-for-paygent-payment-main' )));

		// Actions
		add_action( 'woocommerce_receipt_' . $this->id, array( $this, 'receipt_page' ) );
		add_action( 'woocommerce_update_options_payment_gateways', array( $this, 'process_admin_options' ) );
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		add_action( 'woocommerce_thankyou_' . $this->id, array( $this, 'thankyou_payment_form' ) );
        add_action( 'woocommerce_order_status_completed', array( $this, 'order_mb_status_completed' ) );
	}

	/**
	* Initialize Gateway Settings Form Fields.
	*/
	function init_form_fields() {

		$this->form_fields = array(
			'enabled'     => array(
				'title'       => __( 'Enable/Disable', 'woocommerce-for-paygent-payment-main' ),
				'label'       => sprintf(__( 'Enable paygent %s Payment', 'woocommerce-for-paygent-payment-main' ), __('Carrier Payment', 'woocommerce-for-paygent-payment-main' )),
				'type'        => 'checkbox',
				'description' => '',
				'default'     => 'no'
			),
			'title'       => array(
				'title'       => __( 'Title', 'woocommerce-for-paygent-payment-main' ),
				'type'        => 'text',
				'description' => __( 'This controls the title which the user sees during checkout.', 'woocommerce-for-paygent-payment-main' ),
				'default'     => __( 'Carrier Payment', 'woocommerce-for-paygent-payment-main' )
			),
			'description' => array(
				'title'       => __( 'Description', 'woocommerce-for-paygent-payment-main' ),
				'type'        => 'textarea',
				'description' => __( 'This controls the description which the user sees during checkout.', 'woocommerce-for-paygent-payment-main' ),
				'default'     => sprintf(__( 'Pay with your %s via Paygent.', 'woocommerce-for-paygent-payment-main' ), __('Carrier Payment', 'woocommerce-for-paygent-payment-main' )),
			),
			'order_button_text' => array(
				'title'       => __( 'Order Button Text', 'woocommerce-for-paygent-payment-main' ),
				'type'        => 'textarea',
				'description' => __( 'This controls the description which the user sees during checkout.', 'woocommerce-for-paygent-payment-main' ),
				'default'     => sprintf(__( 'Proceed to %s', 'woocommerce-for-paygent-payment-main' ), __('Carrier Payment', 'woocommerce-for-paygent-payment-main' )),
			),
			'setting_ct_04' => array(
				'id'              => 'wc-paygent-ct-04',
				'type'        => 'checkbox',
				'label'       => __( 'au Easy Payment', 'woocommerce-for-paygent-payment-main' ),
				'default'     => 'yes',
			),
			'setting_ct_05' => array(
				'id'              => 'wc-paygent-ct-05',
				'type'        => 'checkbox',
				'label'       => __( 'Docomo Payment', 'woocommerce-for-paygent-payment-main' ),
				'default'     => 'yes',
			),
			'setting_ct_06' => array(
				'id'              => 'wc-paygent-ct-06',
				'type'        => 'checkbox',
				'label'       => __( 'SoftBank Matomete Payment(B)', 'woocommerce-for-paygent-payment-main' ),
				'default'     => 'yes',
			),
            'debug' => array(
                'title'   => __( 'Debug Mode', 'woocommerce-for-paygent-payment-main' ),
                'type'    => 'checkbox',
                'label'   => __( 'Enable Debug Mode', 'woocommerce-for-paygent-payment-main' ),
                'default' => 'no',
                'description' => __( 'Save debug data using WooCommerce logging.', 'woocommerce-for-paygent-payment-main' ),
            ),
		);
	}

    /**
     * UI - Payment page fields for paygent Payment.
    */
	function payment_fields() {
		// Description of payment method from settings
		if ( $this->description ) { ?>
		<p><?php echo $this->description; ?></p>
		<?php } ?>
		<fieldset  style="padding-left: 40px;">
		<p><?php _e( 'Please select carrier type where you want to pay', 'woocommerce-for-paygent-payment-main' );?></p>
		<?php $this->carrier_type_select(); ?>
		</fieldset>
	<?php }

	function carrier_type_select(){
		?><select name="career_type">
		<?php foreach($this->carrier_types as $num => $value){?>
		<option value="<?php echo $num; ?>"><?php echo $value;?></option>
	<?php }?>
		</select><?php 
	}

    /**
     * Confirmation of mobile terminal.
     */
	public function is_device(){
		$device_info = '';
		// Store the user agent in a variable.
		$ua = $_SERVER['HTTP_USER_AGENT'];
		// Put the UA of the terminal you want to judge on the smartphone in the array
		$spes = array(
			'iPhone',         // Apple iPhone
			'iPod',           // Apple iPod touch
			'Android',        // Android
			'dream',          // Pre 1.5 Android
			'CUPCAKE',        // 1.5+ Android
			'blackberry',     // blackberry
			'webOS',          // Palm Pre Experimental
			'incognito',      // Other iPhone browser
			'webmate'         // Other iPhone browser
		);
		// Put the UA of the terminal you want to judge with the tablet in the array
		$tabs = array(
			'iPad',
			'Android'
		);
		// Put the UA of the terminal you want to judge with a phone call into the array.
		$mbes = array(
			'DoCoMo',
			'KDDI',
			'DDIPOKET',
			'UP.Browser',
			'J-PHONE',
			'Vodafone',
			'SoftBank',
		);
 
		// デバイス変数が空だったら判定する
		if(empty($device_info)){
			// タブレット判定
			foreach($tabs as $tab){
				$str = "/".$tab."/i";
				// ユーザーエージェントにstrが含まれていたら実行する
				if (preg_match($str,$ua)){
					// strがAndroidだったらのモバイル判定を行う。
					if ($str === '/Android/i'){
						// ユーザーエージェントにMobileが含まれていなかったらタブレット
						if (!preg_match("/Mobile/i", $ua)) {
							$device_info = 'tab';
						}else{
						// ユーザーエージェントにMobileが含まれていたらスマートフォン
							$device_info = 'sp';
 						}
 					}else{
 						// Android以外はそのまま結果を返す
 						$device_info = 'tab';
 					}
				}
 			}
 		}

		// デバイス変数が空だったら判定する
		if(empty($device_info)){
			// スマートフォン判定
			foreach($spes as $sp){
				$str = "/".$sp."/i";
				// ユーザーエージェントにstrが含まれていたらスマートフォン
				if (preg_match($str,$ua)){
					$device_info = 'sp';
				}
			}
		}

		// デバイス変数が空だったら判定する
 		if(empty($device_info)){
 			// ガラケー判定
 			foreach($mbes as $mb){
	 			$str = "/".$mb."/i";
	 			// ユーザーエージェントにstrが含まれていたらガラケー
				if (preg_match($str,$ua)){
					if($mb == 'DoCoMo'){
						$device_info = 'mb-docomo';
					}elseif($mb == 'KDDI'){
						$device_info = 'mb-au';
					}elseif($mb == 'SoftBank' or $mb == 'Vodafone' or $mb == 'J-PHONE'){
						$device_info = 'mb-softbank';
					}
	 				
				}
 			}
		}
 
		// If none of the judgments are successful, consider it a PC.
		if(empty($device_info)){
			$device_info = 'pc';
		}
		return $device_info;
	}

	/**
	 * Process the payment and return the result.
     *
     * @param int $order_id
     * @return mixed
	 */
	function process_payment( $order_id ) {

		$order = wc_get_order( $order_id );

		$send_data = array();

		//Common header
		$telegram_kind = '100';
        $prefix_order = get_option( 'wc-paygent-prefix_order' );
        if($prefix_order){
            $send_data['trading_id'] = $prefix_order.$order_id;
        }else{
            $send_data['trading_id'] = 'wc_'.$order_id;
        }
		$send_data['payment_id'] = '100';

		$post_career_type = $this->get_post( 'career_type' );
        if(isset($post_career_type)){
            $send_data['career_type'] = intval($this->get_post( 'career_type' ));
        }else{
            $send_data['career_type'] = get_post_meta($order_id, '_career_type', true);
        }
        $send_data['amount'] = $order->get_total();
		$order->add_meta_data('_career_type', $send_data['career_type'], true);

		if($send_data['career_type'] == 3){
			preg_match("/^.+ser([0-9a-zA-Z]+).*$/", $_SERVER['HTTP_USER_AGENT'], $ua_match);
			$send_data['uid'] = $ua_match[1];
		}
		$send_data['return_url'] = $this->get_return_url( $order );
		$send_data['cancel_url'] = wc_get_cart_url();
		$send_data['other_url'] = wc_get_cart_url();
		if($this->is_device()=='mb-docomo'){
			$send_data['pc_mobile_type'] = '1';
		}elseif($this->is_device()=='mb-au'){
			$send_data['pc_mobile_type'] = '2';
		}elseif($this->is_device()=='mb-softbank'){
			$send_data['pc_mobile_type'] = '3';
		}elseif($this->is_device()=='sp'){
			$send_data['pc_mobile_type'] = '4';
		}else{
			$send_data['pc_mobile_type'] = '0';
		}

		$order_open_id = get_post_meta($order_id, 'open_id', true);
		//career_type(4):au career_type(5):Docomo
		if($send_data['career_type']=='4' or $send_data['career_type']=='5'){
			if(isset($order_open_id) and !empty($order_open_id)){
				$send_data['open_id'] = $order_open_id;
			}else{
				$send_data['redirect_url'] = $this->get_return_url( $order );
				$telegram_kind = '104';
				$response_user = $this->paygent_request->send_paygent_request($this->test_mode, $order, $telegram_kind, $send_data, $this->debug);
				if($response_user['result'] == 0 and $response_user['result_array']){
					$order->add_meta_data( '_pc_mobile_type', wc_clean($send_data['pc_mobile_type'] ), true );
					if($send_data['career_type']=='5'){// Docomo
						$order->add_meta_data( '_open_id_redirect_html', mb_convert_encoding($response_user['result_array'][0]['redirect_html'] ,"UTF-8","SJIS" ) );
                        $order->add_meta_data('_paygent_order_id', $send_data['trading_id'], true );
                        $order->update_status( 'pending', sprintf(__( 'Pending %s', 'woocommerce-for-paygent-payment-main' ), __('Carrier Payment', 'woocommerce-for-paygent-payment-main' ) ) );
						return array (
							'result'   => 'success',
							'redirect' => $this->get_return_url( $order ),
						);
					}else{// au and Softbank, Y!mobile
                        $order->add_meta_data( '_open_id_redirect_url', wc_clean($response_user['result_array'][0]['redirect_url'] ), true );
                        $order->add_meta_data('_paygent_order_id', $send_data['trading_id'], true );
                        $order->update_status( 'pending', sprintf(__( 'Pending %s', 'woocommerce-for-paygent-payment-main' ), __('Carrier Payment', 'woocommerce-for-paygent-payment-main' ) ) );
						return array (
							'result'   => 'success',
							'redirect' => $response_user['result_array'][0]['redirect_url'],
						);
					}
				}else{
					$this->paygent_request->error_response($response_user, $order);
					return array('result' => 'failed');
				}
			}
		}
		$response = $this->paygent_request->send_paygent_request($this->test_mode, $order, $telegram_kind, $send_data, $this->debug);

		// Check response
		if ( $response['result'] == 0 and $response['result_array']) {
            $order->add_meta_data('_paygent_order_id', $send_data['trading_id'], true );
			$this->response_success($order, $order_id, $response);
			// Return thank you redirect
			if(isset($response['result_array'][0]['redirect_url'])){
				return array (
					'result'   => 'success',
					'redirect' => $response['result_array'][0]['redirect_url'],
				);
			}else{
				return array (
					'result'   => 'success',
					'redirect' => $this->get_return_url( $order ),
				);
			}
		}else{
			$this->paygent_request->error_response($response, $order);
			return array('result' => 'failed');
		}
	}

    /**
     * Processing when a successful response comes
     *
     * @param $order
     * @param $order_id
     * @param $response
     */
	function response_success($order, $order_id, $response){
		// Success
		//set transaction id for Paygent Order Number
        $order->set_transaction_id(wc_clean( $response['result_array'][0]['payment_id'] ));
		update_post_meta( $order_id, '_trade_generation_date', wc_clean( $response['result_array'][0]['trade_generation_date'] ) );
		if(isset($response['result_array'][0]['redirect_url'])){
			update_post_meta( $order_id, '_redirect_url', wc_clean($response['result_array'][0]['redirect_url'] ) );
		}else{
			update_post_meta( $order_id, '_redirect_html', mb_convert_encoding($response['result_array'][0]['redirect_html'],"UTF-8","SJIS" ) );
		}
		// Mark as on-hold (we're awaiting the payment)
		$order->update_status( 'on-hold', sprintf(__( 'Awaiting %s', 'woocommerce-for-paygent-payment-main' ), __('Carrier Payment', 'woocommerce-for-paygent-payment-main' ) ) );
	}

    /**
     * Pay Form for thank you page
     *
     * @param int $order_id
     */
	function thankyou_payment_form($order_id) {
        $order = wc_get_order( $order_id );
		$open_id = get_post_meta($order_id, 'open_id', true);
		if(isset($_GET['open_id']) and empty($open_id)){
			$order->add_meta_data( 'open_id', wc_clean( $_GET['open_id'] ), true );
			//Common header
			$telegram_kind = '100';
            $prefix_order = get_option( 'wc-paygent-prefix_order' );
            if($prefix_order){
                $send_data['trading_id'] = $prefix_order.$order_id;
            }else{
                $send_data['trading_id'] = 'wc_'.$order_id;
            }

			$send_data['career_type'] = get_post_meta($order_id, '_career_type', true);
			$send_data['amount'] = $order->get_total();
			$send_data['return_url'] = $this->get_return_url( $order );
			$send_data['cancel_url'] = wc_get_cart_url();
			$send_data['other_url'] = wc_get_cart_url();
			$send_data['pc_mobile_type'] = get_post_meta($order_id, '_pc_mobile_type', true);
			$send_data['open_id'] = $_GET['open_id'];

            if($send_data['career_type'] == 4){
				$send_data['sales_flg'] = 1;
			}
			$response = $this->paygent_request->send_paygent_request($this->test_mode, $order, $telegram_kind, $send_data, $this->debug);
			// Check response
			if ( $response['result'] == 0 and $response['result_array']) {
				$this->response_success($order, $order_id, $response);
				// Return thank you redirect
				if(isset($response['result_array'][0]['redirect_url'])){
					header("Location: " . $response['result_array'][0]['redirect_url']);
				}else{
					header("Location: " . $this->get_return_url( $order ));
				}
			}else{
				$this->paygent_request->error_response($response, $order);
			}
		}
		$status = $order->get_status();
		$javascript_auto_send_code = '
<script type="text/javascript">
function send_form_submit() {
    document.form.submit();
}
window.onload = send_form_submit;
</script>';
		if(isset($_GET['payment_id']) and isset($_GET['career_payment_id']) and isset($_GET['accept_date'])){
			$order->update_status( 'processing', sprintf(__( 'processing this order via %s', 'woocommerce-for-paygent-payment-main' ), __('Carrier Payment', 'woocommerce-for-paygent-payment-main' ) ) );
		}elseif($status == 'on-hold'){
			$form = get_post_meta($order_id, '_redirect_html', true);
			echo $form;
			echo $javascript_auto_send_code;
			echo __('This order was not complete. Please pay from here to Carrier payment', 'woocommerce-for-paygent-payment-main');
		}elseif($status == 'pending'){
			$form = get_post_meta($order_id, '_open_id_redirect_html', true);
			echo $form;
            echo $javascript_auto_send_code;
			echo __('This order was not complete. Please authorised from here to Carrier site', 'woocommerce-for-paygent-payment-main');
		}
	}

	/**
	 * Process a refund if supported
	 * @param  int $order_id
	 * @param  float $amount
	 * @param  string $reason
	 * @return  boolean True or false based on success, or a WP_Error object
	 */
	public function process_refund( $order_id, $amount = null, $reason = '' ) {
        $telegram_array = array(
            'auth_cancel' => '102',
            'sale_cancel' => '102',
            'auth_change' => '103',
            'sale_change' => '103',
        );
        $permit_statuses = array(
            5 => array(//docomo
                'auth_cancel' => array(20,21),
                'sale_cancel' => array(44),
                'auth_change' => array(20,21),
                'sale_change' => array(44),
            ),
            4 => array(//au
                'auth_cancel' => array(20),
                'sale_cancel' => array(40),
                'auth_change' => array(20),
                'sale_change' => array(40),
            ),
            6 => array(//SoftBank(B)
                'auth_cancel' => array(20,21),
                'sale_cancel' => array(40),
                'auth_change' => array(20.21),
                'sale_change' => array(40),
            )
        );
        $send_data_refund = array(
            'amount' => $amount,
        );
        return $this->paygent_request->paygent_process_refund($order_id, $amount, $telegram_array, $permit_statuses, $send_data_refund, $this);
	}
	
	function receipt_page( $order ) {
		echo '<p>' . __( 'Thank you for your order.', 'woocommerce-for-paygent-payment-main' ) . '</p>';
	}

    /**
     * @param $order_id
     */
	function order_mb_status_completed( $order_id ){
        $telegram_kind = '101';
        $this->paygent_request->order_paygent_status_completed($order_id, $telegram_kind, $this);
    }

	/**
	 * Get post data if set
	 */
	private function get_post( $name ) {
		if ( isset( $_POST[ $name ] ) ) {
			return wc_clean($_POST[ $name ]);
		}
		return null;
	}
}

/**
 * Add the gateway to woocommerce
 */
function add_wc_paygent_carrier_gateway( $methods ) {
	$methods[] = 'WC_Gateway_Paygent_Carrier';
	return $methods;
}
add_filter( 'woocommerce_payment_gateways', 'add_wc_paygent_carrier_gateway' );

/**
 * Edit the available gateway to woocommerce
 */
function edit_available_carrier_gateways( $methods ) {
    $currency = get_woocommerce_currency();
	if($currency !='JPY'){
	unset($methods['paygent_mb']);
	}
	return $methods;
}
add_filter( 'woocommerce_available_payment_gateways', 'edit_available_carrier_gateways' );
