<?php
/**
 * Paygent Payment Gateway
 *
 * Functions of a Paygent Payment Gateway.
 *
 * @class 		WC_Gateway_Paygent_Request
 * @version		2.0.7
 * @author		Artisan Workshop
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

use ArtisanWorkshop\WooCommerce\PluginFramework\v2_0_13 as Framework;

use PaygentModule\System\PaygentB2BModule;

/**
 * Generates requests to send to Paygent
 */
class WC_Gateway_Paygent_Request {

    /**
     * Merchant id for Paygent.
     *
     * @var string
     */
    public $merchant_id;

    /**
     * Connect id for Paygent.
     *
     * @var string
     */
    public $connect_id;

    /**
     * Connect password for Paygent.
     *
     * @var string
     */
    public $connect_password;

    /**
     * TEST Merchant id for Paygent.
     *
     * @var string
     */
    public $merchant_test_id;

    /**
     * TEST Connect id for Paygent.
     *
     * @var string
     */
    public $connect_test_id;

    /**
     * TEST Connect password for Paygent.
     *
     * @var string
     */
    public $connect_test_password;

    /**
     * Site id for Paygent.
     *
     * @var integer
     */
    public $site_id;

    /**
     * Prefix Order
     *
     * @var string
     */
    public $prefix_order;

    /**
     * Framework.
     *
     * @var object
     */
    public $jp4wc_framework;

    /**
     * setting data
     *
     * @var array
     */
    public $app;

    /** Socket for SSL communication */
    var $ch;

    /**
	 * Constructor
	 *
	 */
	public function __construct() {
		//Paygent Setting IDs
		$this->merchant_id = get_option('wc-paygent-mid');
		$this->connect_id = get_option('wc-paygent-cid');
		$this->connect_password = get_option('wc-paygent-cpass');
		$this->merchant_test_id = get_option('wc-paygent-test-mid');
		$this->connect_test_id = get_option('wc-paygent-test-cid');
		$this->connect_test_password = get_option('wc-paygent-test-cpass');
		$this->site_id = get_option('wc-paygent-sid');
        $this->prefix_order = get_option('wc-paygent-prefix_order');

        $this->jp4wc_framework = new Framework\JP4WC_Plugin();
	}

	/**
	 * Get the Paygent request Post data for an order
     * @param  boolean  $test_mode
	 * @param  object  $order WC_Order
     * @param  string  $telegram_kind
     * @param  array  $send_data
     * @param  string $debug
	 * @return array
	 */
	public function send_paygent_request($test_mode, $order, $telegram_kind, $send_data, $debug = 'yes') {
		$data = $this->merchant_data($test_mode);

		$process = new PaygentB2BModule();
		$process->init();
		$process->reqPut('merchant_id',$data['merchant_id']);
		$process->reqPut('connect_id',$data['connect_id']);
		$process->reqPut('connect_password',$data['connect_password']);
		$process->reqPut('telegram_kind',$telegram_kind);
		$process->reqPut('telegram_version','1.0');

		// Make Hash check header
		if(get_option( 'wc-paygent-hash_check' )){
			if( $test_mode ){
				$hash_code = get_option( 'wc-paygent-test-hash_code' );
			}else{
				$hash_code = get_option( 'wc-paygent-hash_code' );
			}
			$hash_data = array(
                'merchant_id' => $data['merchant_id'],
                'connect_id' => $data['connect_id'],
				'connect_password' => $data['connect_password'],
				'telegram_kind' => $telegram_kind,
				'telegram_version' => '1.0',
				'trading_id' => $send_data['trading_id'],
				'payment_id' => $send_data['payment_id']
            );
			if(isset($send_data['payment_amount'])){
				$hash_data['payment_amount'] = $send_data['payment_amount'];
			}elseif(isset($send_data['amount'])){
				$hash_data['payment_amount'] = $send_data['amount'];
			}
			$hash_data['request_date'] = date_i18n( 'YmdHis' );
			$send_data['request_date'] = date_i18n( 'YmdHis' );
			$send_data['hc'] = $this->make_hash_data($hash_data, $hash_code);
		}

		// set send_data to reqPut
		foreach($send_data as $key => $value){
			$process->reqPut($key,$value);
		}

        //Save debug send data.
		$send_message = 'telegram_kind : '.$telegram_kind."\n";
		if(!is_null($order)){
            $send_message .= __('This request send data for order ID:', 'woocommerce-for-paygent-payment-main' ).$order->get_id()."\n";
        }
        foreach ($send_data as $key => $value){
            $request_array[$key] = mb_convert_encoding($value, 'UTF-8', 'SJIS');
        }
        $send_message .= __('The request transmission data is shown below.', 'woocommerce-for-paygent-payment-main' )."\n".$this->jp4wc_framework->jp4wc_array_to_message( $request_array );
		$this->jp4wc_framework->jp4wc_debug_log( $send_message, $debug, 'wc-paygent' );

		$process->post();

		$res_array = array();
		while($process->hasResNext()){
			$res_array[] = $process->resNext();
		}

		$result_data = array(
			"result" => $process->getResultStatus(),
			"responseCode" =>$process->getResponseCode(),
			"responseDetail" => $process->getResponseDetail(),
			"result_array" => $res_array
		);

        //Save debug response data.
        $send_message = 'telegram_kind : '.$telegram_kind."\n";
        $send_message .= 'result : '.$result_data['result']."\n";
        if($result_data['result'] != 0){
            $send_message .= 'responseCode : '.$result_data['responseCode']."\n";
            $send_message .= 'responseDetail : '.mb_convert_encoding($result_data['responseDetail'], 'UTF-8', 'SJIS')."\n";
        }
        if(!is_null($order)){
            $send_message .= __('This response data for order ID:', 'woocommerce-for-paygent-payment-main' ).$order->get_id()."\n";
        }
		if(isset($res_array[0])){
			$response_array = array();
			foreach ($res_array[0] as $key => $value){
				$response_array[$key] = mb_convert_encoding($value, 'UTF-8', 'SJIS');
			}
			$send_message .= __('The response transmission data is shown below.', 'woocommerce-for-paygent-payment-main' )."\n".$this->jp4wc_framework->jp4wc_array_to_message( $response_array );
			$this->jp4wc_framework->jp4wc_debug_log( $send_message, $debug, 'wc-paygent' );
		}

        return $result_data;
	}

	/**
	 * Set the Paygent IDs for request
	 * @param  $test_mode
	 * @return array
	 */
	public function merchant_data($test_mode){
		if($test_mode == '1'){
			$data['merchant_id'] = $this->merchant_test_id;
			$data['connect_id'] = $this->connect_test_id;
			$data['connect_password'] = $this->connect_test_password;
		}else{
			$data['merchant_id'] = $this->merchant_id;
			$data['connect_id'] = $this->connect_id;
			$data['connect_password'] = $this->connect_password;
		}
		return $data;
	}

    /**
     * Update Sale from Auth to Paygent System
     *
     * @param int $order_id
     * @param string $telegram_kind
     * @param object $object
     */
    public function order_paygent_status_completed($order_id, $telegram_kind, $object){
        $order = wc_get_order( $order_id );
        $order_payment_method = $order->get_payment_method();
        if($object->paymentaction != 'sale' and $order_payment_method == $object->id){
            $send_data['payment_id'] = $order->get_transaction_id();
            // Set Order ID for Paygent
            $paygent_order_id = $order->get_meta('_paygent_order_id');
//            $order->add_order_note($paygent_order_id);
            if($paygent_order_id){
                $send_data['trading_id'] = $paygent_order_id;
            }else{
                $send_data['trading_id'] = 'wc_'.$order_id;
            }
            // Set Site ID
            if($this->site_id!=1){
                $send_data['site_id'] = $this->site_id;
            }else{
                $send_data['site_id'] = 1;
            }
            $response = $this->send_paygent_request($object->test_mode, $order, $telegram_kind, $send_data, $object->debug);
            if($response['result'] == 0){
                $order->add_order_note( __( 'Success this order set to sale at Paygent.', 'woocommerce-for-paygent-payment-main' ) );
            }else{
                $send_data['trading_id'] = $order_id;//Paidy Payment
                $response_again = $this->send_paygent_request($object->test_mode, $order, $telegram_kind, $send_data, $object->debug);
                if($response_again['result'] == 0){
                    $order->add_order_note( __( 'Success this order set to sale at Paygent.', 'woocommerce-for-paygent-payment-main' ) );
                }else{
                    $order->add_order_note( __( 'Failed this order set to sale at Paygent.', 'woocommerce-for-paygent-payment-main' ) );
                }
            }
        }
    }

    /**
     * Process refund for Paygent.
     * @param int $order_id
     * @param int $amount
     * @param array $telegram_array
     * @param array $permit_statuses
     * @param array $send_data_refund
     * @param object $object
     * @return mixed
     */
    public function paygent_process_refund($order_id, $amount, $telegram_array, $permit_statuses, $send_data_refund, $object){
        $order = wc_get_order( $order_id );
        $telegram_kind_check = '094';
        $transaction_id = $order->get_transaction_id();
        $send_data_check['payment_id'] = $transaction_id;
        // Set Order ID for Paygent
        $paygent_order_id = $order->get_meta('_paygent_order_id');
        if($paygent_order_id){
            $send_data_check['trading_id'] = $paygent_order_id;
        }else{
            $send_data_check['trading_id'] = 'wc_'.$order_id;
        }
        $send_data_refund['payment_id'] = $transaction_id;
        $send_data_refund['trading_id'] = $send_data_check['trading_id'];

        $order_result = $this->send_paygent_request($object->test_mode, $order, $telegram_kind_check, $send_data_check, $object->debug);
        $order_total = $order->get_total();
        if($amount == $order_total ){
            foreach($permit_statuses as $key => $permit_status) {
                if($key == 0 or (isset($order_result['result_array'][0]['career_type']) && $order_result['result_array'][0]['career_type'] == $key)) {
                    if (in_array($order_result['result_array'][0]['payment_status'],$permit_status['auth_cancel']) == true) {
                        $telegram_kind_del = $telegram_array['auth_cancel'];//Authority Cancel
                    } elseif (in_array($order_result['result_array'][0]['payment_status'],$permit_status['sale_cancel']) == true) {
                        $telegram_kind_del = $telegram_array['sale_cancel'];//Sales Cancel
                    } else {
                        $order->add_order_note(__('Failed Refund. ', 'woocommerce-for-paygent-payment-main') . __('Not matched payment_status for refund.', 'woocommerce-for-paygent-payment-main'));
                        return false;
                    }
                }
            }
            $del_result = $this->send_paygent_request($object->test_mode, $order, $telegram_kind_del, $send_data_check, $object->debug);
            if($del_result['result'] === '1'){
                $order->add_order_note( __( 'Failed Refund. ', 'woocommerce-for-paygent-payment-main' ).__( 'Error Code :', 'woocommerce-for-paygent-payment-main' ).$del_result['responseCode'].__( ' Error message :', 'woocommerce-for-paygent-payment-main' ).mb_convert_encoding( $del_result['responseDetail'],"UTF-8","SJIS" ));
                return false;
            }elseif($del_result['result'] === '0'){
                $order->add_order_note( __( 'This order has been successfully refunded by Paygent.', 'woocommerce-for-paygent-payment-main' ));
                return true;
            }else{
                $order->add_order_note( __( 'Failed Refund.', 'woocommerce-for-paygent-payment-main' ));
                return false;
            }
        }elseif($amount < $order_total){
            foreach($permit_statuses as $key => $permit_status){
                if($key == 0 or (isset($order_result['result_array'][0]['career_type']) && $order_result['result_array'][0]['career_type'] == $key)){
                    if(in_array($order_result['result_array'][0]['payment_status'],$permit_status['auth_change']) == true){
                        $telegram_kind_refund = $telegram_array['auth_change'];//Authory Change
                    }elseif(in_array($order_result['result_array'][0]['payment_status'],$permit_status['sale_change']) == true){
                        $telegram_kind_refund = $telegram_array['sale_change'];//Sales Change
                    }else{
                        $order->add_order_note( __( 'Failed Refund. ', 'woocommerce-for-paygent-payment-main' ).__( 'Not matched payment_status for refund.', 'woocommerce-for-paygent-payment-main' ));
                        return false;
                    }
                }
            }
            $refund_result = $this->send_paygent_request($object->test_mode, $order, $telegram_kind_refund, $send_data_refund, $object->debug);
            if($refund_result['result'] === '1'){
                $order->add_order_note( __( 'Failed Refund. ', 'woocommerce-for-paygent-payment-main' ).__( 'Error Code :', 'woocommerce-for-paygent-payment-main' ).$refund_result['responseCode'].__( ' Error message :', 'woocommerce-for-paygent-payment-main' ).mb_convert_encoding( $refund_result['responseDetail'],"UTF-8","SJIS" ));
                return false;
            }elseif($refund_result['result'] === '0'){
                $order->set_transaction_id($refund_result['result_array'][0]['payment_id']);
                $order->save();
                $order->add_order_note( __( 'This order has been successfully partial refunded by Paygent.', 'woocommerce-for-paygent-payment-main' ).sprintf(__( 'payment_id changed from %1$s to %2$s.', 'woocommerce-for-paygent-payment-main' ),$refund_result['result_array'][0]['base_payment_id'] ,$refund_result['result_array'][0]['payment_id'] ));
                return true;
            }else{
                $order->add_order_note( __( 'Failed Refund.', 'woocommerce-for-paygent-payment-main' ));
                return false;
            }
        }
        return false;
    }

    /**
	 * Get the Paygent request URL for an order
	 * @param  Response  $response
     * @param  object  $order
	 * @return string
	 */
	public function error_response($response, $order){
		$order_id = $order->get_id();
		if ( $response['result'] == 1 ) {//System Error
			// Other transaction error
			$order->add_order_note( __( 'paygent Payment failed. Sysmte Error: ', 'woocommerce-for-paygent-payment-main' ) . $response['responseCode'] .':'. mb_convert_encoding($response['responseDetail'],"UTF-8","SJIS" ).':'.'wc_'.$order_id );
			if(is_checkout())wc_add_notice( __( 'Sorry, there was an error: ', 'woocommerce-for-paygent-payment-main' ) . $response['responseCode'] .':'. mb_convert_encoding($response['responseDetail'],"UTF-8","SJIS" ), $notice_type = 'error' );
		} else {
			// No response or unexpected response
			$order->add_order_note( __( "paygent Payment failed. Some trouble happened.", 'woocommerce-for-paygent-payment-main' ). $response['result'] .':'.$response['responseCode'] .':'. mb_convert_encoding($response['responseDetail'],"UTF-8","SJIS").':'.'wc_'.$order_id );
			if(is_checkout())wc_add_notice( __( 'No response from payment gateway server. Try again later or contact the site administrator.', 'woocommerce-for-paygent-payment-main' ), $notice_type = 'error' );
		}
	}

    /**
     * make hash data via hash code
     */
    public function make_hash_data($hash_data, $hash_code){
		$header_text = '';
	    foreach ( $hash_data as $key => $value ) {
		    if(isset($value)){
			    $header_text = $header_text.$value;
		    }
	    }
	    $header_text = $header_text.$hash_code;
		$hc = hash( 'sha256', $header_text );
		return $hc;
    }
}
